@rem = '--*-Perl-*--';
@rem = '
@echo off
if exist ..\common\ratlperl.exe (
    ..\common\ratlperl rereg.bat %*
    goto endofperl
)
if exist ratlperl.exe (
    .\ratlperl rereg.bat %*
    goto endofperl
)
if exist CQperl.exe (
    .\CQperl rereg.bat %*
    goto endofperl
)
echo ERROR: Unable to find ratlperl.exe or CQperl.exe.
goto endofperl
@rem ';

#
# Licensed Materials - Property of IBM
# Rational ClearQuest 
# (C) Copyright IBM Corp. 1997, 2014 All Rights Reserved
# US Government Users Restricted Rights -
# Use, duplication or disclosure restricted by GSA ADP Schedule Contract with IBM Corp.
#
# The source code for this program is not published or otherwise
# divested of its trade secrets, irrespective of what has been
# deposited with the U.S. Copyright Office.
#

#                              rereg.bat
#
# Reregister the ClearQuest installation in the current directory.
#
# Assumptions:
#   The common directory is in the PATH.
#   The current directory is the ClearQuest installation directory.
#

use Cwd;
use File::Find;
use File::Spec;
use Win32::TieRegistry;


sub Usage {
    my(@msg) = @_;
    my $cmd = "rereg.bat";

    my $usage = <<EOF;
Usage: $cmd [-help] [-quiet] [-verbose]
            [-install] [-packages] [-servers] [-all]
EOF

    if(@msg) {
        print "\n$cmd: ERROR: ", @msg, "\n";
        print <<EOF;
$usage
EOF
    } else {
        print <<EOF;

This utility re-registers the ClearQuest installed in the current directory.

$usage
Options:
  -h[elp]       -- Display this help message.
  -q[uiet]      -- minimal output
  -v[erbose]    -- maximum output
  -i[nstall]    -- register the ClearQuest installation directory
  -p[ackages]   -- register packages
  -s[servers]   -- register the ClearQuest COM servers
  -d[atadirect] -- register the DataDirect drivers
  -all          -- do all registrations (default)
EOF
    }
    exit 1;
}


sub process_args {
    my (@args) = @_;
    my @new_args;
    while (@args) {
        $_ = shift (@args);
        if (/^-/i) {
            if (/^-h(elp)?$/) {
                Usage();
            } elsif (/^-all$/) {
                $opt_all = 1;
            } elsif (/^-i(nstall)?$/) {
                $opt_install = 1;
            } elsif (/^-p(ackages)?$/) {
                $opt_packages = 1;
            } elsif (/^-q(uiet)?$/) {
                $opt_quiet = 1;
            } elsif (/^-s(ervers)?$/) {
                $opt_servers = 1;
            } elsif (/^-v(erbose)?$/) {
                $opt_verbose = 1;
            } elsif (/^-d(atadirect)?$/) {
                $opt_datadirect = 1;
            } else {
                Usage ("Unknown option: $_");
            }
        } else {
            push (@new_args, $_);
        }
    }
    @new_args;
}

$opt_all = 0;
$opt_install = 0;
$opt_packages = 0;
$opt_quiet = 0;
$opt_servers = 0;
$opt_verbose = 0;
$opt_datadirect = 0;

@RegSvr32Opts = ('/c', '/s');
$ComSvrOpts = '';
$IntSvrOpts = '';

@ARGV = process_args(@ARGV);

if (! ($opt_install || $opt_packages || $opt_servers || $opt_datadirect)) {
    $opt_all = 1;
}

if ($opt_all) {
    $opt_install = 1;
    $opt_packages = 1;
    $opt_servers = 1;
    $opt_datadirect = 1;
}

if($opt_verbose) {
    @RegSvr32Opts = ();
    $ComSvrOpts = '-v';
    $IntSvrOpts = '/verbose';
}

if($opt_quiet) {
    @RegSvr32Opts = ('/c', '/s');
    $ComSvrOpts = '';
    $IntSvrOpts = '';
}

# Do a spot check on a few key files to see if the current directory looks
# like a ClearQuest install.  Only warn about this, since the registration
# attempt should do as much as possible even in a broken install, but give
# the user an option to stop without making any changes.

my @missing = ();
foreach my $needed ("installutil.exe",
                    "packageutil.exe",
                    "cqole.dll",
                    "cqmainttool.dll",
                    "cqinetlib.dll",
                    "cqintsvr11.exe") {
    unless (-e "$needed") {
        push @missing, $needed;
    }
}
if (scalar(@missing) > 0) {
    print "\n";
    print "WARNING: Cannot find key files: " . join(", ",@missing) . ".\n";
    print "WARNING: The current directory may not contain a complete ClearQuest installation.\n";
    print "WARNING: If you continue, some registration steps may not complete successfully.\n";
    print "\n";
    print "Continue with registration [y/n]? ";
    my $answer = <STDIN>;
    if ($answer !~ /^y/i) {
        exit 1;
    }
    print "\n";
}


# Get the path of the current directory.
$Cwd = getcwd();
$Cwd =~ tr[/][\\];

# Add the current directory to the beginning of the path.
$ENV{PATH} = "$Cwd;$ENV{PATH}";

# Get the current version string, recognizing possible environmental override.
if (defined $ENV{CQ_REGISTRY_VERSION}) {
    $ProductVersion = $ENV{CQ_REGISTRY_VERSION};
    $ProductVersion_Label = ' (from CQ_REGISTRY_VERSION)';
}
else {
    # For development and testing purposes, the following provides a way to
    # use the CQ APIs to get the product version.  This is not normally used
    # since it relies on having a functioning CQPerlExt.pm, which itself
    # depends on the current installation being properly registered.  To use
    # this option, run the command like this:
    #
    #   cd <ClearQuest-home>
    #   ratlperl -s .\rereg.bat -UseClearQuestAPI=1 -- <args>
    #
    if ($::UseClearQuestAPI) {
        # Set CQ_HOME to make sure we get the right CQPerlExt.DLL.
        $ENV{CQ_HOME} = getcwd();

        require CQPerlExt;
        my $ClearQuest = CQClearQuest::Build();
        my $ProdInfo = $ClearQuest->CreateProductInfo();
        $ProductVersion = $ProdInfo->GetProductVersion();
        $ProductVersion_Label = ' (from ClearQuest API)';

        if ($ProductVersion eq '0.0.0') {
            print "ERROR: cannot register with local build product version $ProductVersion\n";
            exit (1);
        }
    }
    else {
        $ProductVersion = '8.0.0';
        $ProductVersion_Label = ' (from rereg)';
    }
}
if ($ProductVersion eq '') {
    $ProductVersion_Label =~ s/\(//;
    $ProductVersion_Label =~ s/\)//;
    print "ERROR: unable to determine product version$ProductVersion_Label\n";
    exit (1);
}

print <<EOF;
Updating the ClearQuest registry keys
  Install directory: $Cwd
  Product version:   ${ProductVersion}${ProductVersion_Label}
EOF

#
# register the DataDirect ODBC drivers that are in ..\common\ODBC .
#
if ($opt_datadirect) {
    $basedir = File::Spec->updir($Cwd);
    $newdir = File::Spec->catdir($basedir, "common", "odbc");

    use Cwd 'abs_path';     # aka realpath()
    $datadirectdir = abs_path($newdir);

    # RATLC01048953: rereg.bat corrupts ODBC driver registration
    # Convert slashes to backslashes
    $datadirectdir =~ s@/@\\@g;

    $odbcinst_reg = "HKEY_LOCAL_MACHINE\\SOFTWARE\\ODBC\\ODBCINST.INI";

    # Reset the driver registry keys to point at the current directory.
    #
    print "\nRegistering the Rational DataDirect drivers in $datadirectdir ...\n";
    $Registry->{$odbcinst_reg .
                "\\IBM Rational Software DB2 Wire Protocol\\"} = {
        "APILevel" => "1",
        "ConnectFunctions" => "YYY",
        "DriverODBCVer" => "3.52",
        "FileUsage" => "0",
        "SQLLevel" => "1",
        "UsageCount" => "1",
        "CPTimeout" => "60",
        "Driver" => "$datadirectdir\\RKDB219.DLL",
        "Setup" => "$datadirectdir\\RKDB219.DLL",
        };
    $Registry->{$odbcinst_reg .
                "\\IBM Rational Software Oracle Wire Protocol\\"} = {
        "APILevel" => "1",
        "ConnectFunctions" => "YYY",
        "DriverODBCVer" => "3.52",
        "FileUsage" => "0",
        "SQLLevel" => "1",
        "UsageCount" => "1",
        "CPTimeout" => "60",
        "Driver" => "$datadirectdir\\RKORA19.DLL",
        "Setup" => "$datadirectdir\\RKORA19S.DLL",
        };
    $Registry->{$odbcinst_reg .
                "\\IBM Rational Software 7.1 DB2 Wire Protocol\\"} = {
        "APILevel" => "1",
        "ConnectFunctions" => "YYY",
        "DriverODBCVer" => "3.52",
        "FileUsage" => "0",
        "SQLLevel" => "1",
        "UsageCount" => "1",
        "CPTimeout" => "60",
        "Driver" => "$datadirectdir\\RKDB227.DLL",
        "Setup" => "$datadirectdir\\RKDB227.DLL",
        };
    $Registry->{$odbcinst_reg .
                "\\IBM Rational Software 7.1 Oracle Wire Protocol\\"} = {
        "APILevel" => "1",
        "ConnectFunctions" => "YYY",
        "DriverODBCVer" => "3.52",
        "FileUsage" => "0",
        "SQLLevel" => "1",
        "UsageCount" => "1",
        "CPTimeout" => "60",
        "Driver" => "$datadirectdir\\RKORA27.DLL",
        "Setup" => "$datadirectdir\\RKORA27.DLL",
        };
    $Registry->{$odbcinst_reg .
                "\\ODBC Drivers\\IBM Rational Software DB2 Wire Protocol"} =
                    "Installed";
    $Registry->{$odbcinst_reg .
                "\\ODBC Drivers\\IBM Rational Software Oracle Wire Protocol"} =
                    "Installed";
    $Registry->{$odbcinst_reg .
                "\\ODBC Drivers\\IBM Rational Software 7.1 DB2 Wire Protocol"} =
                    "Installed";
    $Registry->{$odbcinst_reg .
                "\\ODBC Drivers\\IBM Rational Software 7.1 Oracle Wire Protocol"} =
                    "Installed";
}

if ($opt_install) {

    # Reset the install registry keys to point at the current directory.
    #
    print "\nRegistering the ClearQuest installation directory ...\n";
    $Registry->{'HKEY_LOCAL_MACHINE\\Software\\'} = {
        'Rational Software\\' => {
            'ClearQuest\\' => {
                '\\CurrentVersion' => $ProductVersion,
                $ProductVersion.'\\' => {
                    'Install\\' => {
                        '\\TARGETDIR' => $Cwd,
                        '\\DATADIR' => $Cwd,
                        '\\SRCDIR' => $Cwd,
                        'SqlAnywhere\\' => {
                            '\\BinPath' => $Cwd,
                        }
                    },
                },
            },
        },
    };

    # Invoke installutil to register the installation directory datapath.
    # It registers it under both HKEY_LOCAL_MACHINE and HKEY_CURRENT_USER.
    # Using installutil ensures that we are consistent with other uses of
    # installutil in the harness tests, which affect the same registry
    # entries.
    #
    System(qq(.\\installutil.exe registerdatapath "$Cwd")) &&
        warn("    ERROR: Installation datapath registration failed. Exit status: ", $?>>8, "\n");
}

sub regsvr32 {
    my $rc = System('regsvr32', @RegSvr32Opts, @_);
    if($rc) {
        my($hi, $lo) = ($rc >> 8, $rc & 0xff);
        warn("    ERROR: Registration failed. Exit status: $hi\n");
    }
    $rc;
}

if ($opt_servers) {
    # Reregister the COM objects.

    print "\nRegistering the ClearQuest COM servers ...\n";

    regsvr32(qq(.\\cqole.dll));
    regsvr32(qq(.\\cqmainttool.dll));
    regsvr32(qq(.\\cqaxmform.dll));
#   remove state diagram for now
#   regsvr32(qq(.\\CQDDSShapes.dll));
#   regsvr32(qq(.\\CQStateDiagram.dll));
    regsvr32(qq(.\\CQICQDatastore.dll));
    regsvr32(qq(.\\cqemail.dll));
    regsvr32(qq(.\\cqmail.dll));
    regsvr32(qq(.\\cqmailinter.dll));
    regsvr32(qq(.\\cqratmail.dll));
    regsvr32(qq(.\\cqimportsvr.dll));
    regsvr32(qq(.\\cqinetlib.dll));
    regsvr32(qq(.\\cqaxrform.ocx));
    regsvr32(qq(.\\CQGCtrl.ocx));
    regsvr32(qq(.\\CQWkspTreeControl.ocx));
    # cqvssmgr requires visual source safe to register
#   regsvr32(qq(.\\cqvssmgr.dll));
    # CQMSPInt requires microsoft project to register
#   regsvr32(qq(.\\CQMSPInt.dll));

    System(qq(.\\cqintsvr11.exe $IntSvrOpts /regserver)) && warn("    ERROR: Registration failed. Exit status: ", $?>>8, "\n");
    System(qq(.\\cqintsvr10.exe $IntSvrOpts /regserver)) && warn("    ERROR: Registration failed. Exit status: ", $?>>8, "\n");
}


if ($opt_packages) {
    # Reregister the packages from the current directory.

    print "\nRegistering packages ...\n";

    find(\&wanted, "$Cwd\\packages");
}

sub wanted {
    return 1 if(!/^package\.ini$/i);
    my $dir = $File::Find::dir;
    $dir =~ tr[/][\\];
    System(qq(packageutil registerpackage "$dir"));
}

sub System {
    print join(' ', @_), "\n";
    system(@_);
}

__END__
:endofperl
